/*
 *   3dfb -- a graphical multidimensional filebrowser
 *   written by Tilmann Bitterberg (tilmann@bitterberg.de)
 *   Copyright (C) 1999 by Tilmann Bitterberg
 *   
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public License
 *   as published by the Free Software Foundation; either version 2
 *   of the License, or (at your option) any later version.
 *   
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *   
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *   
 *   Thanks to Max Kliche <max@kliche.org> for support an ideas
 *   Also thanks to the HfT-Stuttgart <http://www.fht-stuttgart.de>
 */
#include "myhdrs.h"

/*
 * How lowercase letters work: 
 * ^^^^^^^^^^^^^^^^^^^^^^^^^^^
 * If a lc letter has to be drawn
 *  - scale is set to 1 
 *  - glScalef is called to make the letters smaller
 *  - glTranslatef is called to move the small letter
 *    to the ground line
 *  - because there's no break; at the end of the case
 *    statement, it falls down and draws a capital letter
 *  - at the end of the switch command, scale is checked and 
 *    the former matrix manipulations are inverted
 *  - scale is set to 0
 *  - count is set to count + 0.8, which is needed, when there
 *    is a \n in the string, because the cursor has to be set
 *    back to the starting X position.
 *
 *    see below, where 0.8 comes from.
 */

float count=0.0;/* how many letters have we drawn */

int drawWord(char *todraw)
{
  int scale=0; /* for lowercase letters this is = 1 */

  count = 0.0;   
  glPushMatrix(); /* Save the current Matrix */
  while (*todraw != '\0' )
  {
    switch (*todraw)
    {
      case '0': z0(); break;
      case '1': z1(); break;
      case '2': z2(); break;
      case '3': z3(); break;
      case '4': z4(); break;
      case '5': z5(); break;
      case '6': z6(); break;
      case '7': z7(); break;
      case '8': z8(); break;
      case '9': z9(); break;
      case 'a': glTranslatef(0.0, -0.5, 0.0); 
		glScalef(0.5, 0.5, 0.0);  scale=1;
      case 'A': A(); break;
      case 'b': glTranslatef(0.0, -0.5, 0.0); 
		glScalef(0.5, 0.5, 0.0);  scale=1;
      case 'B': B(); break;
      case 'c': glTranslatef(0.0, -0.5, 0.0); 
		glScalef(0.5, 0.5, 0.0);  scale=1;
      case 'C': C(); break;
      case 'd': glTranslatef(0.0, -0.5, 0.0);
		glScalef(0.5, 0.5, 0.0);  scale=1;
      case 'D': D(); break;
      case 'e': glTranslatef(0.0, -0.5, 0.0); 
		glScalef(0.5, 0.5, 0.0);  scale=1;
      case 'E': E(); break;
      case 'f': glTranslatef(0.0, -0.5, 0.0);
		glScalef(0.5, 0.5, 0.0);  scale=1;
      case 'F': F(); break;
      case 'g': glTranslatef(0.0, -0.5, 0.0); 
		glScalef(0.5, 0.5, 0.0);  scale=1;
      case 'G': G(); break;
      case 'h': glTranslatef(0.0, -0.5, 0.0);
		glScalef(0.5, 0.5, 0.0);  scale=1;
      case 'H': H(); break;
      case 'i': glTranslatef(0.0, -0.5, 0.0);
		glScalef(0.5, 0.5, 0.0);  scale=1;
      case 'I': I(); break;
      case 'j': glTranslatef(0.0, -0.5, 0.0);
		glScalef(0.5, 0.5, 0.0);  scale=1;
      case 'J': J(); break;
      case 'k': glTranslatef(0.0, -0.5, 0.0);
		glScalef(0.5, 0.5, 0.0);  scale=1;
      case 'K': K(); break;
      case 'l': glTranslatef(0.0, -0.5, 0.0); 
		glScalef(0.5, 0.5, 0.0);  scale=1;
      case 'L': L(); break;
      case 'm': glTranslatef(0.0, -0.5, 0.0);
		glScalef(0.5, 0.5, 0.0);  scale=1;
      case 'M': M(); break;
      case 'n': glTranslatef(0.0, -0.5, 0.0);
		glScalef(0.5, 0.5, 0.0);  scale=1;
      case 'N': N(); break;
      case 'o': glTranslatef(0.0, -0.5, 0.0);
		glScalef(0.5, 0.5, 0.0);  scale=1;
      case 'O': O(); break;
      case 'p': glTranslatef(0.0, -0.5, 0.0);
		glScalef(0.5, 0.5, 0.0);  scale=1;
      case 'P': P(); break;
      case 'q': glTranslatef(0.0, -0.5, 0.0);
		glScalef(0.5, 0.5, 0.0);  scale=1;
      case 'Q': Q(); break;
      case 'r': glTranslatef(0.0, -0.5, 0.0);
		glScalef(0.5, 0.5, 0.0);  scale=1;
      case 'R': R(); break;
      case 's': glTranslatef(0.0, -0.5, 0.0);
		glScalef(0.5, 0.5, 0.0);  scale=1;
      case 'S': S(); break;
      case 't': glTranslatef(0.0, -0.5, 0.0);
		glScalef(0.5, 0.5, 0.0);  scale=1;
      case 'T': T(); break;
      case 'u': glTranslatef(0.0, -0.5, 0.0);
		glScalef(0.5, 0.5, 0.0);  scale=1;
      case 'U': U(); break;
      case 'v': glTranslatef(0.0, -0.5, 0.0);
		glScalef(0.5, 0.5, 0.0);  scale=1;
      case 'V': V(); break;
      case 'w': glTranslatef(0.0, -0.5, 0.0);
		glScalef(0.5, 0.5, 0.0);  scale=1;
      case 'W': W(); break;
      case 'x': glTranslatef(0.0, -0.5, 0.0);
		glScalef(0.5, 0.5, 0.0);  scale=1;
      case 'X': X(); break;
      case 'y': glTranslatef(0.0, -0.5, 0.0);
		glScalef(0.5, 0.5, 0.0);  scale=1;
      case 'Y': Y(); break;
      case 'z': glTranslatef(0.0, -0.5, 0.0);
		glScalef(0.5, 0.5, 0.0);  scale=1;
      case 'Z': Z(); break;
      case '/': Slash(); break;
      case '\\': BSlash(); break;
      case '\n': Newline(); count-=1.0; break;
      case '%': Procent(); break;
      case '+': Plus(); break;
      case '-': Minus(); break;
      case '*': Asterisk(); break;
      case '=': Equal(); break;
      case '$': Dollar(); break;
      case '@': At(); break;
      case '#': Hash(); break;
		case '\"': Dquotes(); break;
      case '\'': Squotes(); break;
      case '`': Bquotes(); break;
      case '.': Dot(); break;
      case ',': Comma(); break;
      case ':': Colon(); break;
      case ';': Semicolon(); break;
      case '!': Emark(); break;
      case '?': Qmark(); break;
      case '(': NbracketL(); break;
      case ')': NbracketR(); break;
      case '{': RbracketL(); break;
      case '}': RbracketR(); break;
      case '[': SbracketL(); break;
      case ']': SbracketR(); break;
      case '<': HbracketL(); break;
      case '>': HbracketR(); break;
      case '|': Pipe(); break;
      case '_': Underscore(); break;
      case '~': Tilde(); break;
      case '^': Caret(); break;
      case '': Degree(); break;
      case '': Paragraph(); break;
      case '&': Reference(); break;
      default: break; /* this is where a blank matches */
    } /* switch end */

    if (scale == 1) {
      /* we have to draw a lowercase letter */
      /* "undo" former matrix manipulations */
      glScalef(2.0, 2.0, 0.0);
      glTranslatef(0.0, 0.5, 0.0);
      scale=0;
      /* move the "cursor" 2.0 further for lowercase letters */
      glTranslatef(2.0, 0.0, 0.0);
      /* 
       * uppercase (2.5) := 1.0
       *            1.0  := (1.0/2.5)
       * lowercase (2.0) := (1.0/2.5)*2.0 = 0.8           
       */
      count += 0.8;
    }
    else
    {
      /* move the "cursor" 2.5 further for uppercase letters */
      glTranslatef(2.5, 0.0, 0.0);
      /* UC letters := 1 */
      count += 1.0;
    }
    todraw++; /* go to next char */ 
  } /* while end */
  glPopMatrix(); /* Restore old current Matrix */
  return (count);
}


/* ****************************************** */

GLvoid z0(GLvoid){ 
  glBegin(GL_LINE_LOOP);
    glVertex2f(1.0, 0.5);
    glVertex2f(0.5, 1.0);
    glVertex2f(-0.5, 1.0);
    glVertex2f(-1.0, 0.5);
    glVertex2f(-1.0, -0.5);
    glVertex2f(-0.5, -1.0);
    glVertex2f(0.5, -1.0);
    glVertex2f(1.0, -0.5);
  glEnd();
  glBegin(GL_LINES);
    glVertex2f(-1.0, -1.0);
    glVertex2f(1.0, 1.0);
  glEnd();
}

/* ****************************************** */

GLvoid z1(GLvoid){ 
  glBegin(GL_LINE_STRIP);
    glVertex2f(-1.0, 0.0);
    glVertex2f(1.0, 1.0);
    glVertex2f(1.0, -1.0);
  glEnd();
}

/* ****************************************** */

GLvoid z2(GLvoid){ 
  glBegin(GL_LINE_STRIP);
    glVertex2f(-1.0, 0.5);
    glVertex2f(-0.5, 1.0);
    glVertex2f(0.5, 1.0);
    glVertex2f(1.0, 0.5);
    glVertex2f(1.0, 0.0);
    glVertex2f(-1.0, -1.0);
    glVertex2f(1.0, -1.0);
  glEnd();
}

/* ****************************************** */

GLvoid z3(GLvoid){ 
  glBegin(GL_LINE_STRIP);
    glVertex2f(-1.0, 0.5);
    glVertex2f(-0.5, 1.0);
    glVertex2f(0.5, 1.0);
    glVertex2f(1.0, 0.5);
    glVertex2f(0.5, 0.0);
    glVertex2f(-0.5, 0.0);
  glEnd();
  glBegin(GL_LINE_STRIP);
    glVertex2f(0.5, 0.0);
    glVertex2f(1.0, -0.5);
    glVertex2f(0.5, -1.0);
    glVertex2f(-0.5, -1.0);
    glVertex2f(-1.0, -0.5);
  glEnd();
}

/* ****************************************** */

GLvoid z4(GLvoid){ 
  glBegin(GL_LINE_STRIP);
    glVertex2f(0.0, 1.0);
    glVertex2f(-1.0, 0.0);
    glVertex2f(1.0, 0.0);
  glEnd();
  glBegin(GL_LINES);
    glVertex2f(0.0, 0.5);
    glVertex2f(0.0, -1.0);
  glEnd();
}

/* ****************************************** */

GLvoid z5(GLvoid){ 
  glBegin(GL_LINE_STRIP);
    glVertex2f(1.0, 1.0);
    glVertex2f(-1.0, 1.0);
    glVertex2f(-1.0, 0.0);
    glVertex2f(0.5, 0.0);
    glVertex2f(1.0, -0.5);
    glVertex2f(0.5, -1.0);
    glVertex2f(-0.5, -1.0);
    glVertex2f(-1.0, -0.5);
  glEnd();
}

/* ****************************************** */

GLvoid z6(GLvoid){ 
  glBegin(GL_LINE_STRIP);
    glVertex2f(1.0, 0.5);
    glVertex2f(0.5, 1.0);
    glVertex2f(-0.5, 1.0);
    glVertex2f(-1.0, 0.5);
    glVertex2f(-1.0, -0.5);
    glVertex2f(-0.5, -1.0);
    glVertex2f(0.5, -1.0);
    glVertex2f(1.0, -0.5);
    glVertex2f(0.5, 0.0);
    glVertex2f(-1.0, 0.0);
  glEnd();
}

/* ****************************************** */

GLvoid z7(GLvoid){ 
  glBegin(GL_LINE_STRIP);
    glVertex2f(-1.0, 1.0);
    glVertex2f(1.0, 1.0);
    glVertex2f(-1.0, -1.0);
  glEnd();
  glBegin(GL_LINES);
    glVertex2f(-1.0, 0.0);
    glVertex2f(1.0, 0.0);
  glEnd();
}

/* ****************************************** */

GLvoid z8(GLvoid){ 
  glBegin(GL_LINE_LOOP);
    glVertex2f(-1.0, 0.5);
    glVertex2f(-0.5, 1.0);
    glVertex2f(0.5, 1.0);
    glVertex2f(1.0, 0.5);
    glVertex2f(0.5, 0.0);
    glVertex2f(-0.5, 0.0);
  glEnd();
  glBegin(GL_LINE_STRIP);
    glVertex2f(-0.5, 0.0);
    glVertex2f(-1.0, -0.5);
    glVertex2f(-0.5, -1.0);
    glVertex2f(0.5, -1.0);
    glVertex2f(1.0, -0.5);
    glVertex2f(0.5, 0.0);
  glEnd();
}

/* ****************************************** */

GLvoid z9(GLvoid){ 
  glBegin(GL_LINE_STRIP);
    glVertex2f(-1.0, -0.5);
    glVertex2f(-0.5, -1.0);
    glVertex2f(0.5, -1.0);
    glVertex2f(1.0, -0.5);
    glVertex2f(1.0, 0.5);
    glVertex2f(0.5, 1.0);
    glVertex2f(-0.5, 1.0);
    glVertex2f(-1.0, 0.5);
    glVertex2f(-0.5, 0.0);
    glVertex2f(1.0, 0.0);
  glEnd();
}

/* ****************************************** */

GLvoid A(GLvoid) { 
  glBegin(GL_LINE_STRIP);
    glVertex2f(-1.0, -1.0); 
    glVertex2f(0.0, 1.0); 
    glVertex2f(1.0, -1.0);
  glEnd();
  glBegin(GL_LINES);
    glVertex2f(-0.5, 0.0); 
    glVertex2f(0.5, 0.0);
  glEnd();
}

/* ****************************************** */

GLvoid B(GLvoid) { 
  glBegin(GL_LINE_STRIP);
    glVertex2f(-1.0, -1.0);
    glVertex2f(-1.0, 1.0);
    glVertex2f(0.0, 1.0);
    glVertex2f(1.0, 0.66);
    glVertex2f(1.0, 0.33);
    glVertex2f(0.0, 0.0);
    glVertex2f(-1.0, 0.0);
  glEnd();
  glBegin(GL_LINE_STRIP);
    glVertex2f(0.0, 0.0);
    glVertex2f(1.0, -0.33);
    glVertex2f(1.0, -0.66);
    glVertex2f(0.0, -1.0);
    glVertex2f(-1.0, -1.0);
  glEnd();
}

/* ****************************************** */

GLvoid C(GLvoid) {
  glBegin(GL_LINE_STRIP);
    glVertex2f(1.0, 0.5);
    glVertex2f(0.5, 1.0);
    glVertex2f(-0.5, 1.0);
    glVertex2f(-1.0, 0.5);
    glVertex2f(-1.0, -0.5);
    glVertex2f(-0.5, -1.0);
    glVertex2f(0.5, -1.0);
    glVertex2f(1.0, -0.5);
  glEnd();
}

/* ****************************************** */

GLvoid D(GLvoid) {
  glBegin(GL_LINE_LOOP);
    glVertex2f(-1.0, 1.0);
    glVertex2f(0.5, 1.0);
    glVertex2f(1.0, 0.5);
    glVertex2f(1.0, -0.5);
    glVertex2f(0.5, -1.0);
    glVertex2f(-1.0, -1.0);
  glEnd();
}

/* ****************************************** */

GLvoid E(GLvoid) {
  glBegin(GL_LINE_STRIP);
    glVertex2f(1.0, 1.0);
    glVertex2f(-1.0, 1.0);
    glVertex2f(-1.0, -1.0);
    glVertex2f(1.0, -1.0);
  glEnd();
  glBegin(GL_LINES);
    glVertex2f(-1.0, 0.0);
    glVertex2f(0.0, 0.0);
  glEnd();
}

/* ****************************************** */

GLvoid F(GLvoid) {
  glBegin(GL_LINE_STRIP);
    glVertex2f(1.0, 1.0);
    glVertex2f(-1.0, 1.0);
    glVertex2f(-1.0, -1.0);
  glEnd();
  glBegin(GL_LINES);
    glVertex2f(-1.0, 0.0);
    glVertex2f(0.0, 0.0);
  glEnd();
}

/* ****************************************** */

GLvoid G(GLvoid) {
  glBegin(GL_LINE_STRIP);
    glVertex2f(1.0, 0.5);
    glVertex2f(0.5, 1.0);
    glVertex2f(-0.5, 1.0);
    glVertex2f(-1.0, 0.5);
    glVertex2f(-1.0, -0.5);
    glVertex2f(-0.5, -1.0);
    glVertex2f(0.5, -1.0);
    glVertex2f(1.0, -0.5);
    glVertex2f(1.0, 0.0);
    glVertex2f(0.0, 0.0);
  glEnd();
}

/* ****************************************** */

GLvoid H(GLvoid) {
  glBegin(GL_LINES);
    glVertex2f(-1.0, -1.0); 
    glVertex2f(-1.0, 1.0); 
    glVertex2f(1.0, -1.0);
    glVertex2f(1.0, 1.0);
    glVertex2f(-1.0, 0.0); 
    glVertex2f(1.0, 0.0);
  glEnd();
}

/* ****************************************** */

GLvoid I(GLvoid) {
  glBegin(GL_LINES);
    glVertex2f(0.0, -1.0); 
    glVertex2f(0.0, 1.0);
    glVertex2f(-0.5, 1.0); 
    glVertex2f(0.5, 1.0); 
    glVertex2f(-0.5, -1.0); 
    glVertex2f(0.5, -1.0); 
  glEnd();
}

/* ****************************************** */

GLvoid J(GLvoid) {
  glBegin(GL_LINE_STRIP);
    glVertex2f(-1.0, 1.0); 
    glVertex2f(1.0, 1.0); 
    glVertex2f(1.0, -0.5); 
    glVertex2f(0.5, -1.0); 
    glVertex2f(-0.5, -1.0); 
    glVertex2f(-1.0, -0.5); 
  glEnd();
}

/* ****************************************** */

GLvoid K(GLvoid) {
  glBegin(GL_LINES);
    glVertex2f(-1.0, 1.0); 
    glVertex2f(-1.0, -1.0); 
    glVertex2f(-1.0, -0.25); 
    glVertex2f(1.0, 1.0); 
    glVertex2f(0.0, 0.25); 
    glVertex2f(1.0, -1.0); 
  glEnd();
}

/* ****************************************** */

GLvoid L(GLvoid) {
  glBegin(GL_LINE_STRIP);
    glVertex2f(-1.0, 1.0); 
    glVertex2f(-1.0, -1.0); 
    glVertex2f(1.0, -1.0); 
  glEnd();
}

/* ****************************************** */

GLvoid M(GLvoid) {
  glBegin(GL_LINE_STRIP);
    glVertex2f(-1.0, -1.0); 
    glVertex2f(-1.0, 1.0); 
    glVertex2f(0.0, 0.0); 
    glVertex2f(1.0, 1.0); 
    glVertex2f(1.0, -1.0); 
  glEnd();
}

/* ****************************************** */

GLvoid N(GLvoid) {
  glBegin(GL_LINE_STRIP);
    glVertex2f(-1.0, -1.0); 
    glVertex2f(-1.0, 1.0); 
    glVertex2f(1.0, -1.0); 
    glVertex2f(1.0, 1.0); 
  glEnd();
}

/* ****************************************** */

GLvoid O(GLvoid) {
  glBegin(GL_LINE_LOOP);
    glVertex2f(1.0, 0.5);
    glVertex2f(0.5, 1.0);
    glVertex2f(-0.5, 1.0);
    glVertex2f(-1.0, 0.5);
    glVertex2f(-1.0, -0.5);
    glVertex2f(-0.5, -1.0);
    glVertex2f(0.5, -1.0);
    glVertex2f(1.0, -0.5);
  glEnd();
}

/* ****************************************** */

GLvoid P(GLvoid) {
  glBegin(GL_LINE_STRIP);
    glVertex2f(-1.0, -1.0);
    glVertex2f(-1.0, 1.0);
    glVertex2f(0.0, 1.0);
    glVertex2f(1.0, 0.66);
    glVertex2f(1.0, 0.33);
    glVertex2f(0.0, 0.0);
    glVertex2f(-1.0, 0.0);
  glEnd();
}

/* ****************************************** */

GLvoid Q(GLvoid) {
  glBegin(GL_LINE_LOOP);
    glVertex2f(1.0, 0.5);
    glVertex2f(0.5, 1.0);
    glVertex2f(-0.5, 1.0);
    glVertex2f(-1.0, 0.5);
    glVertex2f(-1.0, -0.5);
    glVertex2f(-0.5, -1.0);
    glVertex2f(0.5, -1.0);
    glVertex2f(1.0, -0.5);
  glEnd();
  glBegin(GL_LINES);
    glVertex2f(0.0, 0.0);
    glVertex2f(1.0, -1.0);
  glEnd();
}

/* ****************************************** */

GLvoid R(GLvoid) {
  glBegin(GL_LINE_STRIP);
    glVertex2f(-1.0, -1.0);
    glVertex2f(-1.0, 1.0);
    glVertex2f(0.0, 1.0);
    glVertex2f(1.0, 0.66);
    glVertex2f(1.0, 0.33);
    glVertex2f(0.0, 0.0);
    glVertex2f(-1.0, 0.0);
  glEnd();
  glBegin(GL_LINES);
    glVertex2f(0.0, 0.0);
    glVertex2f(1.0, -1.0);
  glEnd();
}

/* ****************************************** */

GLvoid S(GLvoid) {
  glBegin(GL_LINE_STRIP);
    glVertex2f(1.0, 0.66);
    glVertex2f(0.5, 1.0);
    glVertex2f(-0.5, 1.0);
    glVertex2f(-1.0, 0.66);
    glVertex2f(-1.0, 0.33);
    glVertex2f(-0.5, 0.0);
    glVertex2f(0.5, 0.0);
    glVertex2f(1.0, -0.33);
    glVertex2f(1.0, -0.66);
    glVertex2f(0.5, -1.0);
    glVertex2f(-0.5, -1.0);
    glVertex2f(-1.0, -0.66);
  glEnd();
}

/* ****************************************** */

GLvoid T(GLvoid) {
  glBegin(GL_LINES);
    glVertex2f(-1.0, 1.0); 
    glVertex2f(1.0, 1.0); 
    glVertex2f(0.0, -1.0); 
    glVertex2f(0.0, 1.0); 
  glEnd();
}

/* ****************************************** */

GLvoid U(GLvoid) {
  glBegin(GL_LINE_STRIP);
    glVertex2f(-1.0, 1.0);
    glVertex2f(-1.0, -0.5);
    glVertex2f(-0.5, -1.0);
    glVertex2f(0.5, -1.0);
    glVertex2f(1.0, -0.5);
    glVertex2f(1.0, 1.0);
  glEnd();
}

/* ****************************************** */

GLvoid V(GLvoid) {
  glBegin(GL_LINE_STRIP);
    glVertex2f(-1.0, 1.0);
    glVertex2f(0.0, -1.0);
    glVertex2f(1.0, 1.0);
  glEnd();
}

/* ****************************************** */

GLvoid W(GLvoid) {
  glBegin(GL_LINE_STRIP);
    glVertex2f(-1.0, 1.0); 
    glVertex2f(-1.0, -1.0); 
    glVertex2f(0.0, 0.0); 
    glVertex2f(1.0, -1.0); 
    glVertex2f(1.0, 1.0); 
  glEnd();
}

/* ****************************************** */

GLvoid X(GLvoid) {
  glBegin(GL_LINES);
    glVertex2f(-1.0, 1.0); 
    glVertex2f(1.0, -1.0); 
    glVertex2f(1.0, 1.0); 
    glVertex2f(-1.0, -1.0); 
  glEnd();
}

/* ****************************************** */

GLvoid Y(GLvoid) {
  glBegin(GL_LINES);
    glVertex2f(-1.0, 1.0); 
    glVertex2f(0.0, 0.0); 
    glVertex2f(0.0, 0.0); 
    glVertex2f(1.0, 1.0); 
    glVertex2f(0.0, 0.0); 
    glVertex2f(0.0, -1.0); 
  glEnd();
}

/* ****************************************** */

GLvoid Z(GLvoid) {
  glBegin(GL_LINE_STRIP);
    glVertex2f(-1.0, 1.0); 
    glVertex2f(1.0, 1.0); 
    glVertex2f(-1.0, -1.0); 
    glVertex2f(1.0, -1.0); 
  glEnd();
  glBegin(GL_LINES);
    glVertex2f(-0.5, 0.0); 
    glVertex2f(0.5, 0.0);
  glEnd();
}

/* ****************************************** */

GLvoid Slash(GLvoid) { /* / */
  glBegin(GL_LINES);
    glVertex2f(1.0, 1.0); 
    glVertex2f(-1.0, -1.0); 
  glEnd();
}

/* ****************************************** */

GLvoid BSlash(GLvoid) { /* \ */
  glBegin(GL_LINES);
    glVertex2f(-1.0, 1.0); 
    glVertex2f(1.0, -1.0); 
  glEnd();
}

/* ****************************************** */

GLvoid Newline(GLvoid) { /* \n */
  glTranslatef(-2.5*(GLfloat) (count+1.0), -2.5, 0.0);
  count=0.0;
}

/* ****************************************** */

GLvoid Procent(GLvoid) { /* % */
  glBegin(GL_LINES);
    glVertex2f(1.0, 1.0); 
    glVertex2f(-1.0, -1.0); 
  glEnd();
  glBegin(GL_LINE_LOOP);
    glVertex2f(-1.0, 0.66); 
    glVertex2f(-0.66, 1.0); 
    glVertex2f(-0.33, 1.0); 
    glVertex2f(0.0, 0.66); 
    glVertex2f(0.0, 0.33); 
    glVertex2f(-0.33, 0.0); 
    glVertex2f(-0.66, 0.0); 
    glVertex2f(-1.0, 0.33); 
  glEnd();
  glBegin(GL_LINE_LOOP);
    glVertex2f(1.0, -0.66); 
    glVertex2f(0.66, -1.0); 
    glVertex2f(0.33, -1.0); 
    glVertex2f(0.0, -0.66); 
    glVertex2f(0.0, -0.33); 
    glVertex2f(0.33, 0.0); 
    glVertex2f(0.66, 0.0); 
    glVertex2f(1.0, -0.33); 
  glEnd();
}

/* ****************************************** */

GLvoid Plus(GLvoid) { /* + */
  glBegin(GL_LINES);
    glVertex2f(0.0, -0.5); 
    glVertex2f(0.0, 0.5); 
    glVertex2f(-0.5, 0.0); 
    glVertex2f(0.5, 0.0); 
  glEnd();
}

/* ****************************************** */

GLvoid Minus(GLvoid) { /* - */
  glBegin(GL_LINES);
    glVertex2f(-0.5, 0.0); 
    glVertex2f(0.5, 0.0); 
  glEnd();
}

/* ****************************************** */

GLvoid Asterisk(GLvoid) { /* * */
  glBegin(GL_LINES);
    glVertex2f(0.0, -0.5); 
    glVertex2f(0.0, 0.5); 
    glVertex2f(-0.5, 0.33); 
    glVertex2f(0.5, -0.33); 
    glVertex2f(0.5, 0.33); 
    glVertex2f(-0.5, -0.33); 
  glEnd();
}

/* ****************************************** */

GLvoid Equal(GLvoid) { /* = */
  glBegin(GL_LINES);
    glVertex2f(1.0, 0.33); 
    glVertex2f(-1.0, 0.33); 
    glVertex2f(1.0, -0.33); 
    glVertex2f(-1.0, -0.33); 
  glEnd();
}

/* ****************************************** */

GLvoid Dollar(GLvoid) { /* $ */
  glBegin(GL_LINE_STRIP);
    glVertex2f(1.0, 0.66);
    glVertex2f(0.5, 1.0);
    glVertex2f(-0.5, 1.0);
    glVertex2f(-1.0, 0.66);
    glVertex2f(-1.0, 0.33);
    glVertex2f(-0.5, 0.0);
    glVertex2f(0.5, 0.0);
    glVertex2f(1.0, -0.33);
    glVertex2f(1.0, -0.66);
    glVertex2f(0.5, -1.0);
    glVertex2f(-0.5, -1.0);
    glVertex2f(-1.0, -0.66);
  glEnd();
  glBegin(GL_LINES);
    glVertex2f(-0.33, 1.0);
    glVertex2f(-0.33, -1.0);
    glVertex2f(0.33, 1.0);
    glVertex2f(0.33, -1.0);
  glEnd();
}

/* ****************************************** */

GLvoid At(GLvoid) { /* @ */
  glBegin(GL_LINE_LOOP);
    glVertex2f(1.0, 0.0); 
    glVertex2f(0.5, -0.33); 
    glVertex2f(0.0, -0.33); 
    glVertex2f(-0.5, 0.0); 
    glVertex2f(-0.5, 0.33); 
    glVertex2f(0.0, 0.66); 
    glVertex2f(0.5, 0.66); 
    glVertex2f(1.0, 0.33); 
  glEnd();
  glBegin(GL_LINE_STRIP);
    glVertex2f(1.0, 0.33); 
    glVertex2f(1.0, 0.66);
    glVertex2f(0.5, 1.0);
    glVertex2f(-0.5, 1.0);
    glVertex2f(-1.0, 0.5);
    glVertex2f(-1.0, -0.5);
    glVertex2f(-0.5, -1.0);
    glVertex2f(0.5, -1.0);
    glVertex2f(1.0, -0.5);
  glEnd();
}

/* ****************************************** */

GLvoid Hash(GLvoid) { /* # */
  glBegin(GL_LINES);
    glVertex2f(1.0, 0.33); 
    glVertex2f(-1.0, 0.33); 
    glVertex2f(1.0, -0.33); 
    glVertex2f(-1.0, -0.33); 
    glVertex2f(0.66, 1.0); 
    glVertex2f(0.0, -1.0); 
    glVertex2f(0.0, 1.0); 
    glVertex2f(-0.66, -1.0); 
  glEnd();
}

/* ****************************************** */

GLvoid Dquotes(GLvoid) { /* " */
  glBegin(GL_LINE_STRIP);
    glVertex2f(0.33, 1.0); 
    glVertex2f(0.33, 0.33); 
    glVertex2f(0.0, 0.0); 
  glEnd();
  glBegin(GL_LINE_STRIP);
    glVertex2f(0.0, 1.0); 
    glVertex2f(0.0, 0.33); 
    glVertex2f(-0.33, 0.0); 
  glEnd();
}

/* ****************************************** */

GLvoid Squotes(GLvoid) { /* ' */
  glBegin(GL_LINE_STRIP);
    glVertex2f(0.0, 1.0); 
    glVertex2f(0.0, 0.33); 
    glVertex2f(-0.33, 0.0); 
  glEnd();
}

/* ****************************************** */

GLvoid Bquotes(GLvoid) { /* ` */
  glBegin(GL_LINE_STRIP);
    glVertex2f(-0.33, 1.0); 
    glVertex2f(-0.33, 0.33); 
    glVertex2f(0.0, 0.0); 
  glEnd();
}

/* ****************************************** */

GLvoid Dot(GLvoid) { /* . */
  glBegin(GL_LINE_LOOP);
    glVertex2f(-0.25, -1.0); 
    glVertex2f(0.25, -1.0); 
    glVertex2f(0.25, -0.5); 
    glVertex2f(-0.25, -0.5); 
  glEnd();
}

/* ****************************************** */

GLvoid Comma(GLvoid) { /* , */
  glBegin(GL_LINE_STRIP);
    glVertex2f(0.0, -0.33); 
    glVertex2f(0.0, -0.66); 
    glVertex2f(-0.33, -1.0); 
  glEnd();
}

/* ****************************************** */

GLvoid Colon(GLvoid) { /* : */
  glBegin(GL_LINE_LOOP);
    glVertex2f(-0.25, 0.0); 
    glVertex2f(0.25, 0.0); 
    glVertex2f(0.25, 0.5); 
    glVertex2f(-0.25, 0.5); 
  glEnd();
  glBegin(GL_LINE_LOOP);
    glVertex2f(-0.25, -1.0); 
    glVertex2f(0.25, -1.0); 
    glVertex2f(0.25, -0.5); 
    glVertex2f(-0.25, -0.5); 
  glEnd();
}

/* ****************************************** */

GLvoid Semicolon(GLvoid) { /* ; */
  glBegin(GL_LINE_LOOP);
    glVertex2f(-0.25, 0.0); 
    glVertex2f(0.25, 0.0); 
    glVertex2f(0.25, 0.5); 
    glVertex2f(-0.25, 0.5); 
  glEnd();
  glBegin(GL_LINE_STRIP);
    glVertex2f(0.0, -0.33); 
    glVertex2f(0.0, -0.66); 
    glVertex2f(-0.33, -1.0); 
  glEnd();
}

/* ****************************************** */

GLvoid Emark(GLvoid) { /* ! */
  glBegin(GL_LINES);
    glVertex2f(0.0, 1.0); 
    glVertex2f(0.0, -0.33); 
  glEnd();
  glBegin(GL_LINE_LOOP);
    glVertex2f(-0.25, -1.0); 
    glVertex2f(0.25, -1.0); 
    glVertex2f(0.25, -0.5); 
    glVertex2f(-0.25, -0.5); 
  glEnd();
}

/* ****************************************** */

GLvoid Qmark(GLvoid) { /* ? */
  glBegin(GL_LINE_STRIP);
    glVertex2f(-1.0, 0.66);
    glVertex2f(-0.5, 1.0);
    glVertex2f(0.5, 1.0);
    glVertex2f(1.0, 0.66);
    glVertex2f(0.5, 0.33);
    glVertex2f(0.0, 0.33);
    glVertex2f(0.0, -0.33);
  glEnd();
  glBegin(GL_LINE_LOOP);
    glVertex2f(-0.25, -1.0); 
    glVertex2f(0.25, -1.0); 
    glVertex2f(0.25, -0.5); 
    glVertex2f(-0.25, -0.5); 
  glEnd();
}

/* ****************************************** */

GLvoid NbracketL(GLvoid) { /* ( */ 
  glBegin(GL_LINE_STRIP);
    glVertex2f(0.5, 1.0);
    glVertex2f(0.0, 0.5);
    glVertex2f(0.0, -0.5);
    glVertex2f(0.5, -1.0);
  glEnd();
}

/* ****************************************** */

GLvoid NbracketR(GLvoid) { /* ) */ 
  glBegin(GL_LINE_STRIP);
    glVertex2f(-0.5, 1.0);
    glVertex2f(0.0, 0.5);
    glVertex2f(0.0, -0.5);
    glVertex2f(-0.5, -1.0);
  glEnd();
}

/* ****************************************** */

GLvoid RbracketL(GLvoid) { /* { */ 
  glBegin(GL_LINE_STRIP);
    glVertex2f(0.5, 1.0);
    glVertex2f(0.0, 1.0);
    glVertex2f(0.0, 0.33);
    glVertex2f(-0.5, 0.0);
    glVertex2f(0.0, -0.33);
    glVertex2f(0.0, -1.0);
    glVertex2f(0.5, -1.0);
  glEnd();
}

/* ****************************************** */

GLvoid RbracketR(GLvoid) { /* } */ 
  glBegin(GL_LINE_STRIP);
    glVertex2f(-0.5, 1.0);
    glVertex2f(0.0, 1.0);
    glVertex2f(0.0, 0.33);
    glVertex2f(0.5, 0.0);
    glVertex2f(0.0, -0.33);
    glVertex2f(0.0, -1.0);
    glVertex2f(-0.5, -1.0);
  glEnd();
}

/* ****************************************** */

GLvoid SbracketL(GLvoid) { /* [ */ 
  glBegin(GL_LINE_STRIP);
    glVertex2f(0.5, 1.0);
    glVertex2f(0.0, 1.0);
    glVertex2f(0.0, -1.0);
    glVertex2f(0.5, -1.0);
  glEnd();
}

/* ****************************************** */

GLvoid SbracketR(GLvoid) { /* ] */ 
  glBegin(GL_LINE_STRIP);
    glVertex2f(-0.5, 1.0);
    glVertex2f(0.0, 1.0);
    glVertex2f(0.0, -1.0);
    glVertex2f(-0.5, -1.0);
  glEnd();
}

/* ****************************************** */

GLvoid HbracketL(GLvoid) { /* < */ 
  glBegin(GL_LINE_STRIP);
    glVertex2f(0.5, 1.0);
    glVertex2f(-0.5, 0.0);
    glVertex2f(0.5, -1.0);
  glEnd();
}

/* ****************************************** */

GLvoid HbracketR(GLvoid) { /* > */ 
  glBegin(GL_LINE_STRIP);
    glVertex2f(-0.5, 1.0);
    glVertex2f(0.5, 0.0);
    glVertex2f(-0.5, -1.0);
  glEnd();
}

/* ****************************************** */

GLvoid Pipe(GLvoid) { /* | */ 
  glBegin(GL_LINES);
    glVertex2f(0.0, 1.0);
    glVertex2f(0.0, -1.0);
  glEnd();
}

/* ****************************************** */

GLvoid Underscore(GLvoid) { /* _ */ 
  glBegin(GL_LINES);
    glVertex2f(-1.0, -1.0);
    glVertex2f(1.0, -1.0);
  glEnd();
}

/* ****************************************** */

GLvoid Tilde(GLvoid) { /* ~ */ 
  glBegin(GL_LINE_STRIP);
    glVertex2f(-1.0, -0.165);
    glVertex2f(-0.66, 0.165);
    glVertex2f(-0.33, 0.165);
    glVertex2f(0.33, -0.165);
    glVertex2f(0.66, -0.165);
    glVertex2f(1.0, 0.165);
  glEnd();
}

/* ****************************************** */

GLvoid Caret(GLvoid) { /* ^ */ 
  glBegin(GL_LINE_STRIP);
    glVertex2f(1.0, 0.5);
    glVertex2f(0.0, 1.0);
    glVertex2f(-1.0, 0.5);
  glEnd();
}

/* ****************************************** */

GLvoid Degree(GLvoid) { /*  */ 
  glBegin(GL_LINE_LOOP);
    glVertex2f(-0.25, 0.5); 
    glVertex2f(0.25, 0.5); 
    glVertex2f(0.25, 1.0); 
    glVertex2f(-0.25, 1.0); 
  glEnd();
}

/* ****************************************** */

GLvoid Paragraph(GLvoid) { /*  */ 
  glBegin(GL_LINE_LOOP);
    glVertex2f(1.0, -0.165); 
    glVertex2f(0.5, -0.5); 
    glVertex2f(-0.5, -0.5); 
    glVertex2f(-1.0, -0.165); 
    glVertex2f(-1.0, 0.165); 
    glVertex2f(-0.5, 0.5); 
    glVertex2f(0.5, 0.5); 
    glVertex2f(1.0, 0.165); 
  glEnd();
  glBegin(GL_LINE_STRIP);
    glVertex2f(1.0, 0.66);
    glVertex2f(0.5, 1.0);
    glVertex2f(-0.5, 1.0);
    glVertex2f(-1.0, 0.5);
    glVertex2f(-1.0, 0.165);
  glEnd();
  glBegin(GL_LINE_STRIP);
    glVertex2f(1.0, -0.165);
    glVertex2f(1.0, -0.5);
    glVertex2f(0.5, -1.0);
    glVertex2f(-0.5, -1.0);
    glVertex2f(-1.0, -0.66);
  glEnd();
}

/* ****************************************** */

GLvoid Reference(GLvoid) { /* & */ 
  glBegin(GL_LINE_STRIP);
    glVertex2f(1.0, -1.0);
    glVertex2f(-1.0, 0.33); 
    glVertex2f(-1.0, 0.66); 
    glVertex2f(-0.5, 1.0); 
    glVertex2f(-0.0, 1.0); 
    glVertex2f(0.33, 0.66); 
    glVertex2f(0.33, 0.33); 
    glVertex2f(-1.0, -0.33); 
    glVertex2f(-1.0, -0.66); 
    glVertex2f(-0.5, -1.0); 
    glVertex2f(0.33, -1.0); 
    glVertex2f(1.0, 0.0); 
  glEnd();
}
